@@ -23,6 +23,7 @@
 
 
 #include <math.h>
+#include <stdio.h>
 
 /************************************************************************
  * noop. For testing and timing.                                        *
@@ -412,15 +413,17 @@ float ran1()
 }
 
 void _poidev(float *xmv, long n)
+/* all floats -> doubles on June 2010 to avoid SIGFPE
+   for too large input values */
 { 
-  float gammln(float xx); 
+  double gammln(double xx); 
   /*  float ran1(long *idum);*/
-  static float sq,alxm,g,oldm=(-1.0); 
-  float xm,em,t,y,y1; 
+  static double sq,alxm,g,oldm=(-1.0); 
+  double xm,em,t,y,y1; 
   long i;
 
   for (i=0;i<n;i++) {
-    xm = xmv[i];
+    xm = (double)xmv[i];
     if (xm == 0.0f) continue;
     if (xm < 20.0) { /* Use direct method. */ 
       if (xm != oldm) { 
@@ -435,26 +438,26 @@ void _poidev(float *xmv, long n)
       } while (t > g);
     } else {  /* Use rejection method. */ 
       if (xm != oldm) { 
-	oldm=xm; 
+	oldm=xm;
 	sq=sqrt(2.0*xm); 
 	alxm=log(xm); 
 	g=xm*alxm-gammln(xm+1.0); 
       } 
       do { 
 	do { 
-	  y=tan(3.141592654*ran1());
+	  y=tan(3.1415926535897932384626433832*ran1());
 	  em=sq*y+xm; 
 	} while (em < 0.0); 
 	em=floor(em); 
 	t=0.9*(1.0+y*y)*exp(em*alxm-gammln(em+1.0)-g); 
       } while (ran1() > t); 
     } 
-    xmv[i] = em;
+    xmv[i] = (float)em;
   }
 } 
 
 
-float gammln(float xx) 
+double gammln(double xx) 
 { 
   /* Returns the value ln[?(xx)] for xx>0. */
   double x,y,tmp,ser; 
